# -*- coding: utf-8 -*-
import os
import datetime
from django.db import models
from django.template.defaultfilters import slugify
from django.conf import settings
from django.utils.translation import ugettext_lazy as _
from django.utils import timezone

from easy_thumbnails.files import get_thumbnailer

from jmb.core.db.fields import HTMLField, HTMLFieldTinyMCE
from jmb.core.db.models import StatusModel, DateModel, UserModel, OrderedModel, SEOModel
from image_cropping import ImageRatioField


class Category(StatusModel, DateModel, UserModel, OrderedModel):
    title = models.CharField(
        _("Title"),
        max_length=200
    )
    slug = models.SlugField(
        _("Slug"),
        unique=True,
    )

    class Meta:
        verbose_name = _('category')
        verbose_name_plural = _('categories')
        ordering = ["ordering", "title"]

    def __unicode__(self):
        return u"{0}".format(self.title)


class Item(StatusModel, DateModel, UserModel, OrderedModel, SEOModel):
    tale = models.CharField(
        max_length=100,
        null=True, blank=True,
        default=settings.SHOWCASE_ITEM_TALE_DEFAULT,
        help_text=_("in future we may have more tales for showing dynamic content"),
    )
    title = models.CharField(
        _("Title"),
        max_length=200
    )
    slug = models.SlugField(
        _("Slug")
    )
    author = models.CharField(
        _("Author"),
        max_length=200,
        blank=True, null=True,
    )
    source = models.CharField(
        _("Source"),
        max_length=200,
        blank=True, null=True,
    )
    language = models.CharField(
        _("Language"),
        choices=settings.LANGUAGES,
        blank=True, null=True,
        max_length=5,
    )
    content = HTMLFieldTinyMCE(
        _("Content"),
        blank=True, null=True,
        help_text=_("Full description"),
    )
    image = models.ImageField(
        _("Image"),
        blank=True, null=True,
        upload_to=settings.SHOWCASE_ITEM_IMAGE_UPLOAD_TO,
        help_text=_("if 'show_image' is checked, this one will be showed")
    )
    rollover_image = models.ImageField(
        _("rollover Image"),
        blank=True, null=True,
        upload_to=settings.SHOWCASE_ITEM_IMAGE_UPLOAD_TO,
        help_text=_("rollover Image")
    )
    small_image = ImageRatioField(
        'image',
        settings.SHOWCASE_ITEM_SMALL_IMAGE_CROPPING,
        size_warning=True
    )
    large_image = ImageRatioField(
        'image',
        settings.SHOWCASE_ITEM_LARGE_IMAGE_CROPPING,
        size_warning=True
    )
    rollover_small_image = ImageRatioField(
        'rollover_image',
        settings.SHOWCASE_ITEM_SMALL_IMAGE_CROPPING,
        size_warning=True
    )
    icon = ImageRatioField(
        'image',
        settings.SHOWCASE_ITEM_ICON_CROPPING,
        size_warning=True,
        help_text=_("Use only with thumb icon template")
    )
    link = models.URLField(
        _("Link"),
        blank=True, null=True,
        help_text=_("if link_type is external content, this link will be opened")
    )
    internal_link = models.CharField(
        _("Internal link"),
        blank=True, null=True,
        max_length=255,
        help_text=_("use for link internal pages")
    )
    linkedin_link = models.URLField(
        _("linkedin Link"),
        blank=True, null=True,
        help_text=_("displayed only in thumb flat template")
    )
    twitter_link = models.URLField(
        _("twitter Link"),
        blank=True, null=True,
        help_text=_("displayed only in thumb flat template")
    )
    file = models.FileField(
        _("File"),
        blank=True, null=True,
        upload_to=settings.SHOWCASE_ITEM_FILE_UPLOAD_TO,
        help_text=_("if link_type is downloadable content, this item will be downloaded")
    )
    publication_date = models.DateField(
        _("Publication date"),
        default=timezone.now
    )
    active_class = models.BooleanField(
        _("Active class"),
        default=False,
        help_text=_("apply active class on item (only for thumb icon template)")
    )
    show_image = models.BooleanField(
        _("Show image"),
        default=True
    )
    show_link = models.BooleanField(
        _("Show link"),
        default=True
    )
    link_type = models.CharField(
        _("Link type"),
        max_length=3,
        choices=settings.SHOWCASE_LINK_TYPES,
        default='int'
    )
    categories = models.ManyToManyField(
        Category,
        db_index=True,
        related_name="items",
        verbose_name=_("categories"),
    )

    class Meta:
        verbose_name = _('item')
        verbose_name_plural = _('items')
        ordering = ["title", "-publication_date"]

    def __unicode__(self):
        return u"{0}".format(self.title)

    def get_categories(self):
        return ", ".join([category.title for category in self.categories.all().order_by('ordering')])
    get_categories.short_description = _("Categories")

    def has_image(self):
        try:
            self.image.file
            return True
        except:
            return False

    def get_small_image(self):
        if self.image:
            return get_thumbnailer(
                self.image
            ).get_thumbnail({
                'size': (430, 360),
                'box': self.small_image,
                'crop': True,
                'detail': True,
            }).url

    def get_large_image(self):
        if self.image:
            return get_thumbnailer(
                self.image
            ).get_thumbnail({
                'size': (800, 400),
                'box': self.large_image,
                'crop': True,
                'detail': True,
            }).url

# import cms stuffs
if settings.CMS_ENABLED:
    import jmb.showcase.cms.plugins
    import jmb.showcase.cms.app
